import java.io.*;
import java.util.*;

import org.jdom2.*;



/**
 * Takes all given .gazebo.xacro files, extracts their sensors and creates sensorconfig files.
 */
public class SensorConfigSetup extends XMLHelper{

	/**
	 * Obtains an attribute value of the input/ output entries specified in modelsetup.config.
	 * 
	 * @param doc		The JDOM Document (modelsetup.config)
	 * @param entry		String formed as "{input or output}/{entity}"
	 * @param val		Attribute name as String
	 * @return path as specified in modelsetup.config
	 */
	private static String getConfigVal(Document doc, String str, String val) {
		List<Attribute> valList = xQueryAttributes(doc, "//"+str+"/@"+val);
		if(valList.size()!=1) {
			System.err.println("Error while reading input/ output "+val+" value from modelsetup.config.");
			System.exit(-1);
		}

		return valList.get(0).getValue();
	}
	
	/**
	 * Check whether the given directory exists and if not, create it.
	 * 
	 * @param path		Path to directory as String
	 */
	private static void createDirectory(String path) {
		File directory = new File(path);
		if(!directory.exists()) {
			boolean created = directory.mkdir();
			if(!created) {
				System.err.println("Error: Directory \'"+directory.getPath()+"\' does not exist and could not be created.");
				System.exit(-1);
			}
		}
	}
	
	/**
	 * Recursively delete a directory and its contents.
	 * 
	 * @param path		Path to directory as String
	 */
	private static void deleteDirectory(String path) {
		File directory = new File(path);
		File[] content = directory.listFiles();
		if(content != null) {
			for(File file : content) {
				deleteDirectory(file.getPath());
			}
		}
		if(!directory.delete()) {
			System.err.println("Error: Could not delete file/ directory: \'"+directory.getPath()+"\'.");
		}

	}
	
	/**
	 * Call this to run SensorConfigSetup.
	 */
	public SensorConfigSetup() {
		System.out.println("SensorConfigSetup started.");
		
		try {
			final String SC_ENDING = ".xml";
			final String GX_ENDING = ".gazebo.xacro";
			
			//######### Input ###############################			
			Document ModelSetupConfig = readXml("./modelsetup.config");
			
			//Get modelsetup.config values.
			String in_scPath = getConfigVal(ModelSetupConfig, "input/sensorconfig", "path");///TODO
			String in_gxPath = getConfigVal(ModelSetupConfig, "input/gazebo_xacro", "path");
			String out_mPath = getConfigVal(ModelSetupConfig, "output/model", "path");
			
			//Declare Document.
			//Document xy = 
			
			//Get sensorconfig- and gazebomodel file names.
			//in_gxPath may contain other files that will be filtered out!
			List<String> gxDirectoryFilesList = getFileNames(in_gxPath);
			List<String> gxList = new ArrayList<String>();
			for(String s : gxDirectoryFilesList) {
				if(s.endsWith(GX_ENDING)) {
					gxList.add(s);
				}
			}
			
			//Check whether input was empty.
			if(gxList.isEmpty()) {
				System.err.println("Error: No gazebo model \'"+GX_ENDING+"\' files found in \'"+in_gxPath+"\'. Could not autogenerate sensorconfig files");
				System.exit(-1);
			}
			//###############################################
			
			
			//######### Check directories ###################
			String scAuto = in_scPath+"autogenerated/";
			String scOther = in_scPath+"other/";
			
			createDirectory(in_scPath);
			if((new File(scAuto)).exists()) {
				deleteDirectory(scAuto);
			}
			createDirectory(scAuto);
			createDirectory(scOther);
			//###############################################
			
			
			//######### Generate sensorconfigs ##############

			//###############################################
			
			
			//######### Output result #######################
			//saveAsXml(model, out_mPath+out_mName);
			//###############################################
		}
		catch (JDOMException | IOException e) {
			e.printStackTrace();
		}

		System.out.println("SensorConfigSetup finished.");
	}
}



